# procedure utili in TCL
# Paolo Mairano 2008

# restituisce $numero approssimato a $valore, es.: 'approssima 12 5' restituisce 10
proc approssima {numero valore} {
	expr round($numero / $valore) * $valore
}

# restituisce una lista con tutti i valori dell'array passato: opposto di 'array names'
proc array_values {nome} {
	upvar $nome arr
	foreach {key value} [array get arr] {
		lappend result $value
	}
	return $result
}

# resituisce quante volte una stringa ($item)  presente in una lista
proc string_count {lista item} {
	set conto 0
	while {[lsearch $lista $item] >= 0} {
		incr conto; lset lista [lsearch $lista $item] ""
	}
	return $conto
}

# resituisce il valore piu basso fra quelli di una lista
proc min {lista} {
	if {$lista eq ""} {return 0}
	set ris [lindex $lista 0]
	foreach num [lrange $lista 1 end] {
		if {$num < $ris} {set ris $num}
	}
	return $ris
}

# restituisce il valore piu alto fra quelli di una lista
proc max {lista} {
	if {$lista eq ""} {return 0}
	set ris [lindex $lista 0]
	foreach num [lrange $lista 1 end] {
		if {$num > $ris} {set ris $num}
	}
	return $ris
}

# resituisce la somma di tutti i valori in una lista
proc sum {addendi} {
	if {$addendi eq ""} {return 0}
	return [expr [join $addendi +]]
}

# resituisce la media dei valori di una lista
proc mean {lista} {
	if {$lista eq ""} {return 0}
	return [expr ([join $lista +]) / [llength $lista].0]
}

# restituisce la deviazione standard dei valori di una lista
proc stdev {lista} {
	if {$lista eq ""} {return 0}
	set media [mean $lista]
	foreach item $lista {
		lappend provvisorie [expr pow(($item - $media), 2)]
	}
	return [expr sqrt( ([join $provvisorie +]) / ([llength $provvisorie].0 - 1) )]
}

# restituisce la deviazione standard dei valori di una lista
proc stderr {lista} {
	if {$lista eq ""} {return 0}
	if {[llength $lista] < 2} {return 0}
	return [expr [stdev $lista] / sqrt([llength $lista])]
}

# restituisce il varco dei valori di una lista
proc varco {lista} {
	if {$lista eq ""} {return 0}
	return [expr [stdev $lista] / [mean $lista] * 100]
}

# restituisce l'rpvi dei valori di una lista
proc rpvi {valori} {
	set volte 0
	foreach v $valori {
		if {$volte > 0} {
			set prec [lindex $valori [expr $volte - 1]]
			lappend rpvi [expr abs($v - $prec)]
		}
		incr volte
	}
	return [mean $rpvi]
}

# resituisce l'npvi dei valori di una lista
proc npvi {valori} {
	set volte 0
	foreach v $valori {
		if {$volte > 0} {
			set prec [lindex $valori [expr $volte - 1]]
			lappend npvi [expr (abs($v - $prec)) / (($v + $prec) / 2.0)]
		}
		incr volte
	}
	return [expr 100 * [mean $npvi]]
}

# restituisce il cci di una lista di durate e una lista parallela con il numero di segmenti per ogni durata
proc cci {durate segmenti} {
	set volte 0
	foreach durata $durate {
		lappend differenze [expr $durata / [string length [lindex $segmenti $volte]].0]
		incr volte
	}
	set volte 0
	foreach v $differenze {
		if {$volte > 0} {
			set prec [lindex $differenze [expr $volte - 1]]
			lappend cci [expr abs($v - $prec)]
		}
		incr volte
	}
	return [mean $cci]
}

# rinomina un file di testo - "file rename" non funziona su Windows
proc file_rename {vecchionome nuovonome {force no}} {
	if {[file exists $nuovonome] && ($force ne "force")} {return 0}
	if [catch {set vecchio [open $vecchionome r]}] {cannot read $vecchionome; return 0}
	set contenuto [read $vecchio]
	close $vecchio
	if [catch {set nuovo [open $nuovonome w]}] {cannot write $nuovonome; return 0}
	puts $nuovo $contenuto
	close $nuovo
	file delete $vecchionome
	return 1
}

# clona un file di testo - "file copy" funziona solo se il target e' una diversa directory
proc file_clone {vecchionome nuovonome {force no}} {
	if {[file exists $nuovonome] && ($force ne "force")} {return 0}
	if [catch {set vecchio [open $vecchionome r]}] {cannot read $vecchionome; return 0}
	set contenuto [read $vecchio]
	close $vecchio
	if [catch {set nuovo [open $nuovonome w]}] {cannot write $nuovonome; return 0}
	puts $nuovo $contenuto
	close $nuovo
	return 1
}

# legge nel file di configurazione il valore di $item;
# deve esistere una variabile globale di nome configfile con il path completo del file di configurazione,
# oppure e' possibile specificare il path completo di un altro file come secondo parametro della procedura
proc rconfigure {item {file ""}} {
	if {$file eq ""} {global configfile; set file $configfile}
	if {![file exists $file]} {return ""}
	if [catch {open $file r} f] {return ""}
	while {[gets $f riga] >= 0} {
		if {[string match $item* $riga]} {
			set riga [split $riga \t]
			set valore [lindex $riga 1]
			close $f
			return $valore
		}
	}
	return ""
}

# scrive nel file di configurazione il $valore di $item (v. sopra)
proc wconfigure {item valore {file ""}} {
	if {$file eq ""} {global configfile; set file $configfile}
	#se il file non esiste, lo crea, scrive $item\t$valore e ritorna
	if ![file exists $file] {
		if [catch {open $file w} f] {
			return 0
		} else {
			puts $f "$item\t$valore"
			close $f
			return 1
		}
	}
	#legge il file e lo riapre in scrittura, scrive, e richiude
	if [catch {open $file r} f] {
		return 0
	} else {
		set vecchio [read $f]
		close $f
		if [catch {open $file w} f] {
			return 0
		} else {
			set fatto 0
			foreach riga [split $vecchio \n] {
				if {[string match "$item\t*" $riga]} {
					puts $f "$item\t$valore"
					incr fatto
				} elseif {$riga eq ""} {
					continue
				} else {puts $f $riga}
			}
			if {$fatto == 0} {puts $f "$item\t$valore"}
			close $f
			return 1
		}
	}
}

# se una stringa contiene caratteri pericolosi restituisce 0, altrimenti 1
proc checkstring {stringa} {
	if [regexp {.*(\\|\[|\]|\{|\}|\"| ).*} $stringa] {return 0} else {return 1}
}

# restituisce quante occorrenze di una stringa ci sono in un altra
proc string_count_occurrences {pattern stringa} {
	set pos 0
	set conto -1
	while {$pos >= 0} {
		incr conto
		set pos [string first $pattern $stringa [expr $pos + 1]]
	}
	return $conto
}

proc float {n} {
	return [expr double($n)]
}
